@@ -37,24 +37,56 @@
 ##############################################################################
 
 importBardShape <-
-function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE) {
-  filen<-sub("\\.shp","",filen)
+function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE,  projection=as.character(NA),...) {
+  buildIndex<-TRUE
+  filen<-sub("\\.shp$","",filen,ignore.case=TRUE)
   #migrate to spatial data frames
-  tmp.shape<-try( readShapePoly(filen) )
+  tmp.shape<-try( readShapePoly(filen,proj4string=CRS(projection),...) )
   
   if (inherits(tmp.shape,"try-error")) {
       return(NULL)
   }
+
+  #check projection and reproject if possible
+  canTransform<-function() {
+	mrequire<-require
+	retval <- mrequire("rgdal",quietly=TRUE)
+	return(retval)
+  }
+  
+ if (length(grep("+proj=", projection, ignore.case = TRUE)) < 1) {
+    	    print("Note: you have not specified a map projection. Please ensure that the projection you use provides reasonable representation of area, perimeter, and shape at district scale.")
+  }
+
+  tmp.longlat<-FALSE
+  if (length(grep("+proj=latlon",tmp.shape@proj4string@projargs,ignore.case=TRUE)>0) || length(grep("+proj=longlat",tmp.shape@proj4string@projargs,ignore.case=TRUE)>0)) {
+  	  if ( canTransform() ) {
+  	  	  print ("Projecting map using albers equal area projection. This may take some time...")
+  	  	  print(Sys.time())
+  	 
+  	  	  bb<-bbox(tmp.shape)
+  	  	  newprojstring <- paste("+proj=aea", sep="") 
+  	  	#newprojstring <- paste("+proj=aea ","+lat_1=",bb[1,1]," +lat_2=",bb[1,2]," +lat_0=",mean(bb[1,])," +lon_0=",mean(bb[2,])," +x_0=0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs", sep="") 
+  	  	tmp.shape2<-spTransform(tmp.shape, CRS(newprojstring))
+  	  	if (inherits(tmp.shape2,"try-error")) {
+  	  		warning("transformation failed")
+  	  		tmp.longlat<-TRUE
+      		} else {
+      			tmp.shape<-tmp.shape2
+      		}
+  	  }
+  } 
+ 
+
+  if (tmp.longlat) {
+  	   warning("Using unprojected lat-lon coordinates. Area measures may be inaccurate. Consider installing (http://cran.r-project.org/web/packages/rgdal/) to have BARD re-project the map or transforming it with a separate program. The Albers Equal Areas (aea) projection with state-plane and state scale data in meter units is recommended.")
+  }
+
+  
   tmp.polys <-tmp.shape@polygons
 
   tmp.df <- tmp.shape@data
   
-  tmp.bboxs<-sapply(tmp.polys,function(x)bbox(x))
-  tmp.centroids<-sapply(tmp.polys,function(x)x@labpt)
-  # HMM -- performance of this is 1000x slower ?
-  #tmp.bboxs<-sapply(1:dim(tmp.shape)[1],function(x)bbox(tmp.shape[x,]))
-  #tmp.centroids<-sapply(1:dim(tmp.shape)[1],function(x)coordinates(tmp.shape[x,]))
-
   
   mapid <- NULL 
   if ("ID" %in% names(tmp.df)) {
@@ -64,8 +96,7 @@ function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE)
   }
 
 
-  
-  # Note: poly2nb is very slow, so read from a file if present
+  # Note: poly2nb is slow (even after my patches were integrated) so read from a file if present
 
     
   tmp.nb <- try(read.gal(gal))
@@ -77,12 +108,37 @@ function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE)
   
   # not an else, since top condition may change
   if (is.null(gal)) {
-    print("Generating gal file...")
-    tmp.nb <- poly2nb(tmp.shape)
+  	  
+    estmin <- round((length(tmp.polys)*.002)^2 / 60,2)
+    
+
+    print(paste("Since no GAL file supplied, generating gal file, this may take a while  ... (estimated ",estmin," minutes)", sep=" "))
+    print(paste("Starting", Sys.time(),sep=" "))
+    tmp.nb<-myPoly2nb(tmp.polys)    
+    print(paste("Ending", Sys.time(),sep=" "))
+
+
+    
   }
   
-  tmp.timestamp<-Sys.time()
-  retval<-list(shape=tmp.shape,polys=tmp.polys,nb=tmp.nb,df=tmp.df,centroids=tmp.centroids, bboxs=tmp.bboxs, timestamp=tmp.timestamp)
+  retval<-list(shape=tmp.shape,polys=tmp.polys,nb=tmp.nb,df=tmp.df)
+
+  if (buildIndex) {
+    estmin <- round((length(tmp.polys)*.05) / 60,2)
+    print(Sys.time())
+    print(paste("Building indices which will save a lot of time in perimeter calculations later ... ( estimated ",estmin," minutes)", sep=" "))
+    print(paste("Starting", Sys.time(),sep=" "))
+
+  }
+  # generate shape info
+  
+  tmp.info<-genBlockShapeInfo(retval,buildIndex=buildIndex)
+  
+  if (buildIndex) {
+  	      print(paste("Ending", Sys.time(),sep=" "))
+  }
+  retval<-c(retval,tmp.info)
+  
   class(retval)<-"bardBasemap"
   if (wantplan) {
     tmp.plan <- NULL
@@ -102,6 +158,7 @@ function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE)
 
 
 
+
 ##############################################################################
 #
 # exportBardShape
@@ -125,7 +182,7 @@ function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE)
 
 exportBardShape <-
 function(plan,filen,id="BARDPlanID",gal=paste(filen,".GAL",sep="")) {
-  filen<-sub("\\.shp","",filen)
+  filen<-sub("\\.shp$","",filen,ignore.case=TRUE)
     basemap<-basem(plan)
   BARDplan <- as.vector(plan)
   basemap$shape@data[id]<-BARDplan
@@ -162,6 +219,7 @@ function(filen, basemaps=NULL, plans=NULL) {
     if (is.null(basemaps) && is.null(plans)) {
       warning("Saving empty image")
     }
+
     filen<-paste(filen,"_bard_image.Rdata",sep="")
     saveres <- try(save(file=filen, list=c("basemaps","plans")))
     retval<- !inherits(saveres,"try-error")
@@ -170,32 +228,47 @@ function(filen, basemaps=NULL, plans=NULL) {
 
 readBardImage <-
 function(filen) {
+    filen<-sub("_bard_image\\.Rdata$","",filen,ignore.case=TRUE)
     filen<-paste(filen,"_bard_image.Rdata",sep="")
-    retval<-local({
+    loadval<-local({
       basemaps<-NULL
       plans<-NULL
       loadres<-try(load(filen))
       if (inherits(loadres,"try-error")) {
         lrval <- NULL
       } else {
+      	    
 	if (is.null(basemaps) && is.null(plans)) {
 		# failed to load correct values 
+		warning("failed to read image")
 		lrval<-NULL
 	} else {
-		# check to convert from old formats
-		if (!is.null(basemaps)) {
-			basemaps<-sapply(basemaps,convertBaseMap.old,simplify=FALSE)
-		}
-		if (!is.null(plans)) {
-			plans<-sapply(plans,
-			function(x){ basem(x)<-convertBaseMap.old(basem(x)); x},
-			simplify=FALSE)
-		}
         	lrval <- list(basemaps=basemaps,plans=plans)
 	}
       }
       lrval
     })
+    basemaps<-loadval$basemaps
+    plans<-loadval$plans
+    rm("loadval")
+    if (inherits(basemaps,"bardBasemap")) {
+		basemaps<-list(basemaps)
+    }
+    if (inherits(plans,"bardPlan")) {
+		plans<-list(plans)
+    }
+		
+    # check to convert from old formats
+    if (!is.null(basemaps)) {
+		basemaps<-sapply(basemaps,convertBaseMap.old,simplify=FALSE)
+    }
+    if (!is.null(plans)) {
+		plans<-sapply(plans,
+		function(x){ basem(x)<-convertBaseMap.old(basem(x)); x},
+		simplify=FALSE)
+    }
+    retval <- list(basemaps=basemaps,plans=plans)
+
     return(retval)
 }
 
@@ -244,6 +317,7 @@ function(filen, restart.fun=NULL ) {
 
 readBardCheckpoint <-
 function(filen, continue=TRUE) {
+  filen<-sub("_bard_checkpoint\\.Rdata$","",filen, ignore.case=TRUE)
   filen<-paste(filen,"_bard_checkpoint.Rdata",sep="")
   loadres<-try(load(filen, .GlobalEnv))
   retval <- !inherits(loadres,"try-error")
@@ -259,6 +333,89 @@ function(filen, continue=TRUE) {
 }
 
 
+###
+###  pre-calculate areas and perimeters
+###
+
+myUnionSpatialPolygons<-local( {
+	calledPermit <-FALSE
+	function(...) {
+		if (!calledPermit) {
+			if (exists("gpclibPermit")) { gpclibPermit() }
+			calledPermit<-TRUE
+		}
+		return(unionSpatialPolygons(...))
+	}
+})
+
+
+genBlockShapeInfo<-function(basemap, buildIndex=TRUE) {
+ if (length(grep("+proj=latlon",basemap$shape@proj4string@projargs,ignore.case=TRUE)>0) || length(grep("+proj=longlat",basemap$shape@proj4string@projargs,ignore.case=TRUE)>0))  { tmp.longlat<-TRUE} else {tmp.longlat<-FALSE}
+ 
+ 
+   
+   tmp.perims<-sapply(1:length(basemap$polys), function(x)genPerim(basemap$shape[x,],longlat=tmp.longlat))
+   
+   # calculate areas of blocks
+   area<-function(poly)sum(unlist(sapply(poly@polygons, 
+   function(x)sapply(x@Polygons,function(x)x@area))))
+   
+   tmp.areas<-sapply(1:length(basemap$polys), function(x)area(basemap$shape[x,]))
+   
+   tmp.bboxs<-sapply(basemap$polys,function(x)bbox(x))
+  tmp.centroids<-sapply(basemap$polys,function(x)x@labpt)
+  # HMM -- performance of this is 1000x slower ?
+  #tmp.bboxs<-sapply(1:dim(tmp.shape)[1],function(x)bbox(tmp.shape[x,]))
+  #tmp.centroids<-sapply(1:dim(tmp.shape)[1],function(x)coordinates(tmp.shape[x,]))
+  
+
+  
+  if (buildIndex) {
+   # calculate overlapping perim between each connected block
+     basemap$longlat<-tmp.longlat
+     basemap$perims<-tmp.perims
+     sharelist<-genPerimIndex(basemap)
+  } else {
+    sharelist<-NULL
+  }
+     tmp.timestamp<-Sys.time()
+
+   return(list(perims=tmp.perims,sharedPerims=sharelist, areas=tmp.areas, bboxs=tmp.bboxs, perims=tmp.perims, centroids=tmp.centroids, timestamp=tmp.timestamp, longlat=tmp.longlat))
+}
+
+
+# calculate individual block perims
+genPerim<-function(poly,longlat=FALSE) {
+	sum(sapply(poly@polygons, 
+   function(x)sapply(x@Polygons,function(x)LineLength(x@coords,longlat=longlat))))
+}
+  
+genPerimIndex<-function(basemap) {
+
+   sharelist <- unclass(basemap$nb);   attributes(sharelist)<-NULL
+   tmp.longlat<-basemap$longlat
+   tmp.perims<-basemap$perims
+   
+   for ( i in seq(1,length.out=length(basemap$polys)) ) {
+   	   tmplist=sharelist[[i]]
+   	   for (j in seq(1,length.out=length(tmplist))) {
+   	   	 tn <- tmplist[j]
+   	   	 if (tn>=i) {
+   	   	 	usub <- myUnionSpatialPolygons(basemap$shape[c(i,tn),], c(1,1))
+   	   	 	totalperim <- genPerim(usub,longlat=tmp.longlat)
+   	   	 	sharedperim <-  tmp.perims[i] + tmp.perims[tn] - totalperim 
+   	   	 	tmplist[j]<-sharedperim
+   	   	 } else {
+   	   	 	 tmplist[j] <- sharelist[[tn]][which(basemap$nb[[tn]] %in% i)] 
+   	   	 }
+   	   }
+   	   sharelist[[i]]<-tmplist
+   }	
+
+   return(sharelist)
+}
+
+
 ###
 ###  Functions to deal with the conversion of basemaps using the old 
 #### polylist structure
@@ -266,7 +423,7 @@ function(filen, continue=TRUE) {
 
 importBardShape.polyList <-
 function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE) {
-  filen<-sub("\\.shp","",filen)
+  filen<-sub("\\.shp$","",filen,ignore.case=TRUE)
   #migrate to spatial data frames
   ow<-options(warn=-1)
   tmp.shape<-try( maptools:::read.shape(filen) )
@@ -299,11 +456,10 @@ function(filen, id="BARDPlanID", gal=paste(filen,".GAL",sep=""), wantplan=FALSE)
   # not an else, since top condition may change
   if (is.null(gal)) {
     print("Generating gal file...")
-    tmp.nb <- poly2nb(tmp.polys)
+    tmp.nb <- myPoly2nb(tmp.polys)
   }
   
-  tmp.timestamp<-Sys.time()
-  retval<-list(shape=tmp.shape,polys=tmp.polys,nb=tmp.nb,df=tmp.df,timestamp=tmp.timestamp)
+    retval<-list(shape=tmp.shape,polys=tmp.polys,nb=tmp.nb,df=tmp.df)
   class(retval)<-"bardBasemap"
   if (wantplan) {
     tmp.plan <- NULL
@@ -334,7 +490,7 @@ isOldBasemap<-function(basemap) {
 
 exportBardShape.polylist <-
 function(plan,filen,id="BARDPlanID",gal=paste(filen,".GAL",sep="")) {
-  filen<-sub("\\.shp","",filen)
+  filen<-sub("\\.shp$","",filen,ignore.case=TRUE)
     basemap<-basem(plan)
   BARDplan <- as.vector(plan)
   newDf <- cbind(basemap$df,BARDplan)
@@ -356,7 +512,7 @@ convertBaseMap.polylist<-function(basemap) {
 	if (!isOldBasemap(basemap)) {
 		return(basemap)
 	}
-	warning("Attempting to convert basemap from previous polyList representation.")
+	print("Attempting to convert basemap from previous polyList representation.")
 	fn.tmp <- tempfile()
 	plan.tmp<- createAssignedPlan(basemap,replicate(length(basemap$polys),1))
 	exportBardShape.old(plan.tmp, fn.tmp,id="TMPID")
@@ -369,7 +525,23 @@ convertBaseMap.polylist<-function(basemap) {
 
 	return(retval)
 }
-convertBaseMap.old<-convertBaseMap.polylist		
+convertBaseMap.old<-function(basemap) {
+	basemap<-convertBaseMap.polylist(basemap)
+	
+	
+	if(is.null(basemap$longlat)) {
+		print("Map built in a previous version, adding indices, please wait...")
+		tmpinfo<-genBlockShapeInfo(basemap,buildIndex=TRUE)
+		basemap<-c(basemap,tmpinfo)
+		class(basemap)<-"bardBasemap"
+	}
+	
+	return(basemap)
+}
+
+
+
+
 
 ###
 ###  Generic methods
@@ -419,3 +591,4 @@ as.data.frame.bardBasemap<-function (x, row.names = NULL, optional = FALSE, ...)
 dim.bardBasemap<-function(x) {
 	dim(x$shape)
 }
+
